@@ -10,6 +10,7 @@
 """Console script for raytraverse."""
 import os
 import shutil
+import sys
 from glob import glob
 
 import numpy as np
@@ -59,11 +60,11 @@ def main(ctx, out=None, config=None, n=None,  **kwargs):
 
         raytraverse --template > run.cfg
 
-    after adjusting the settings, than each command can be invoked in turn and
+    after adjusting the settings, then each command can be invoked in turn and
     any dependencies will be loaded with the correct options, a complete run
     and evaluation can then be called by::
 
-        raytraverse -c run.cfg skyrun sunrun
+        raytraverse -c run.cfg skyrun sunrun evaluate
 
     as all required precursor commands will be invoked automatically as needed.
     """
@@ -91,8 +92,7 @@ def main(ctx, out=None, config=None, n=None,  **kwargs):
 def scene(ctx, out=None, opts=False, debug=False, version=None, **kwargs):
     """define scene files for renderer and output directory
 
-    Effects
-    ~~~~~~~
+    Effects:
         - creates outdir and outdir/scene.oct
     """
     if out is not None:
@@ -109,12 +109,7 @@ engine_opts = [
                    ' directions with a contribution variance greater than .25'),
  click.option('-idres', default=32,
               help='the initial directional sampling resolution '
-                   '(as sqrt of samples per hemisphere)'),
- click.option('-rayargs',
-              help='additional arguments to pass to the  rendering engine'),
- click.option('--default-args/--no-default-args', default=True,
-              help='use raytraverse defaults before -rayargs, if False, uses'
-                   ' radiance defaults'),
+                   '(as sqrt of samples per hemisphere)')
     ]
 
 
@@ -153,8 +148,7 @@ def area(ctx, static_points=None, zone=None, opts=False, debug=False,
          version=None, printdata=False, printlevel=None, **kwargs):
     """define sampling area
 
-    Effects
-    ~~~~~~~
+    Effects:
         - None
 
     """
@@ -229,8 +223,7 @@ def suns(ctx, loc=None, opts=False, debug=False, version=False, epwloc=False,
          printdata=False, printlevel=None, **kwargs):
     """define solar sampling space
 
-    Effects
-    ~~~~~~~
+    Effects:
         - None
     """
     if hasattr(loc, "shape"):
@@ -296,15 +289,19 @@ def skydata(ctx, wea=None, name="skydata", loc=None, reload=True, opts=False,
             printdata=False, printfull=False, debug=False, version=None, **kwargs):
     """define sky conditions for evaluation
 
-    Effects
-    ~~~~~~~
+    Effects:
         - Invokes scene
         - write outdir/name.npz (SkyData initialization object)
     """
     if 'scene' not in ctx.obj:
         clk.invoke_dependency(ctx, scene, reload=True, overwrite=False)
     scn = ctx.obj['scene']
-    file = f"{scn.outdir}/{name}.npz"
+    if scn.outdir is None:
+        print("Warning! attempting to use skydata from outside scene, "
+              "make sure scene -outdir is set", file=sys.stderr)
+        file = f"{name}.npz"
+    else:
+        file = f"{scn.outdir}/{name}.npz"
     reloaded = reload and os.path.isfile(file)
     if reloaded:
         wea = file
@@ -314,7 +311,10 @@ def skydata(ctx, wea=None, name="skydata", loc=None, reload=True, opts=False,
         loc = (loc[0], loc[1], int(loc[2]))
     sd = SkyData(wea, loc=loc, **kwargs)
     if not reloaded:
-        sd.write(name, scn)
+        if scn.outdir is None:
+            sd.write(name)
+        else:
+            sd.write(name, scn)
     ctx.obj['skydata'] = sd
     if printdata:
         ris = np.arange(sd.skydata.shape[0])
@@ -330,9 +330,24 @@ def skydata(ctx, wea=None, name="skydata", loc=None, reload=True, opts=False,
 
 @main.command()
 @clk.shared_decs(engine_opts)
+@click.option('-rayargs',
+              help='additional arguments to pass to the rendering engine')
+@click.option('--default-args/--no-default-args', default=True,
+              help='use raytraverse defaults before -rayargs, if False, uses'
+                   ' radiance defaults. defaults are: -u+ -ab 16 -av 0 0 0 '
+                   '-aa 0 -as 0 -dc 1 -dt 0 -lr -14 -ad adpatch*(skyres^2+1) '
+                   '-lw 0.008/(skyres^2+1)/adpatch -st 0 -ss 16 -c 1. note that'
+                   ' if this is false -ad and -lw will not be automatically '
+                   'set')
 @click.option("-skyres", default=15,
               help="resolution of sky patches (sqrt(patches / hemisphere))."
                    "Must match argument givein to skydata")
+@click.option("-adpatch", default=50,
+              help="prefered instead of -ad/-lw in rayargs to better coordinate"
+                   " settings of ad/lw and skypatch division, consider doubling"
+                   " this with each halving of accuracy and in cases with high"
+                   " proportion indirect contributions, such as deep spaces or"
+                   " complex fenestrations")
 @click.option('-nlev', default=5,
               help='number of directional sampling levels, yielding a final'
                    'resolution of idres^2 * 2^(nlev) samples per hemisphere')
@@ -350,8 +365,7 @@ def skyengine(ctx, accuracy=1.0, vlt=0.64, idres=32, rayargs=None,
               usedecomp=False, opts=False, debug=False, version=None, **kwargs):
     """initialize engine for skyrun
 
-    Effects
-    ~~~~~~~
+    Effects:
         - Invokes scene
         - creates outdir/scene_sky.oct
     """
@@ -372,6 +386,14 @@ def skyengine(ctx, accuracy=1.0, vlt=0.64, idres=32, rayargs=None,
 
 @main.command()
 @clk.shared_decs(engine_opts)
+@click.option('-rayargs', default="-ab 0",
+              help='additional arguments to pass to the rendering engine,'
+                   ' by default sets -ab 0, pass "" to clear')
+@click.option('--default-args/--no-default-args', default=True,
+              help='use raytraverse defaults before -rayargs, if False, uses'
+                   ' radiance defaults. defaults are: -u+ -ab 16 -av 0 0 0 '
+                   '-aa 0 -as 0 -dc 1 -dt 0 -lr -14 -ad 1000 -lw 0.00004 -st 0 '
+                   '-ss 16 -w-')
 @click.option('-nlev', default=6,
               help='number of directional sampling levels, yielding a final'
                    'resolution of idres^2 * 2^(nlev) samples per hemisphere')
@@ -386,8 +408,7 @@ def sunengine(ctx, accuracy=1.0, vlt=0.64, idres=32, rayargs=None,
               debug=False, version=None, **kwargs):
     """initialize engine for sunrun
 
-    Effects
-    ~~~~~~~
+    Effects:
         - Invokes scene
     """
     if 'scene' not in ctx.obj:
@@ -404,6 +425,13 @@ def sunengine(ctx, accuracy=1.0, vlt=0.64, idres=32, rayargs=None,
 
 @main.command()
 @clk.shared_decs(engine_opts)
+@click.option('-rayargs',
+              help='additional arguments to pass to the rendering engine')
+@click.option('--default-args/--no-default-args', default=True,
+              help='use raytraverse defaults before -rayargs, if False, uses'
+                   ' radiance defaults. defaults are: -u+ -ab 16 -av 0 0 0 '
+                   '-aa 0 -as 0 -dc 1 -dt 0 -lr -14 -ad 1000 -lw 0.00004 -st 0 '
+                   '-ss 16 -w-')
 @click.option("-srcfile", default=None, type=click.Path(dir_okay=False),
               help="scene source description (required)")
 @click.option("-source", default="source", help="name for this source")
@@ -423,8 +451,7 @@ def sourceengine(ctx, srcfile=None, source="source", accuracy=1.0, vlt=1.0,
                  opts=False, debug=False, version=None, **kwargs):
     """initialize engine for sunrun
 
-    Effects
-    ~~~~~~~
+    Effects:
         - Invokes scene
     """
     if srcfile is None:
@@ -437,6 +464,8 @@ def sourceengine(ctx, srcfile=None, source="source", accuracy=1.0, vlt=1.0,
         ctx.obj.pop('sunengine')
     scn = ctx.obj['scene']
     srcscn = scn.source_scene(srcfile, source)
+    if rayargs is None:
+        rayargs = ""
     rayargs += f" -af {scn.outdir}/{source}.amb"
     rtrace = Rtrace(rayargs=rayargs, scene=srcscn, default_args=default_args)
     if color:
@@ -452,7 +481,7 @@ sample_opts = [
               help="parameter to set threshold at sampling level relative to "
                    "final level threshold (smaller number will increase "
                    "sampling)"),
- click.option("-edgemode", default='constant',
+ click.option("-edgemode", default='reflect',
               type=click.Choice(['constant', 'reflect', 'nearest', 'mirror',
                                  'wrap']),
               help="if 'constant' value is set to -self.t1, so edge is "
@@ -479,8 +508,7 @@ def skyrun(ctx, accuracy=1.0, nlev=3, jitter=True, overwrite=False, plotp=False,
            edgemode='constant', opts=False, debug=False, version=None):
     """run scene under sky for a set of points (defined by area)
 
-    Effects
-    ~~~~~~~
+    Effects:
         - Invokes scene
         - Invokes area (no effects)
         - Invokes skyengine
@@ -574,8 +602,7 @@ def sunrun(ctx, srcaccuracy=1.0, srcnlev=3, srcjitter=True, recover=False,
     """run scene for a set of suns (defined by suns) for a set of points
     (defined by area)
 
-    Effects
-    ~~~~~~~
+    Effects:
         - Invokes scene
         - Invokes area (no effects)
         - Invokes sunengine (no effects)
@@ -644,8 +671,7 @@ def sourcerun(ctx, accuracy=1.0, nlev=3, jitter=True, overwrite=False,
     - Do not run as part of the same call as sunrun
     - make sure rayargs are properly set in sunengine (not -ab 0)
 
-    Effects
-    ~~~~~~~
+    Effects:
         - Invokes scene
         - Invokes area (no effects)
         - Invokes sunengine (no effects)
@@ -701,7 +727,8 @@ eval_opts = [
                    "between components of a point and spaces between points. "
                    "points should either all be 3 componnent (x,y,z) or 6"
                    " component (x,y,z,dx,dy,dz). If 3 component, -sdirs is "
-                   "required, if 6-component, -sdirs is ignored"),
+                   "required, if 6-component, -sdirs is ignored. leave as None"
+                   " for zonal evaluation (sdirs required)"),
  click.option("-sdirs", callback=np_load,
               help="sensor directions, this can be a .npy file, a whitespace "
                    "seperated text file or entered as a string with commas "
@@ -760,7 +787,7 @@ def images(ctx, sensors=None, sdirs=None, viewangle=180., skymask=None,
            directview=False, maskfull=True, resamprad=0.0, **kwargs):
     """render images
 
-    Prequisites:
+    Prerequisites:
 
         - skyrun and sunrun must be manually invoked prior to this
 
@@ -884,7 +911,7 @@ def evaluate(ctx, sensors=None, sdirs=None, viewangle=180., skymask=None,
              maskfull=True, resamprad=0.0, **kwargs):
     """evaluate metrics
 
-    Prequisites:
+    Prequisites
 
         - skyrun and sunrun must be manually invoked prior to this
 
@@ -940,7 +967,7 @@ def evaluate(ctx, sensors=None, sdirs=None, viewangle=180., skymask=None,
         result = LightResult(data, result[0].axes[0], ptaxis, result[0].axes[3])
     else:
         if zonal and sdirs is not None:
-            sensors = pm
+            sensors = None
         elif sdirs is None:
             raise ValueError("if sensors do not have directions, sdirs cannot be "
                              "None")
