@@ -5,7 +5,9 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 # =======================================================================
+import os
 import sys
+import tempfile
 
 import numpy as np
 
@@ -132,10 +134,35 @@ class Integrator(object):
                                      resamprad=resamprad)
         return sorted([i for j in outfs for i in j])
 
+    def _echunk(self, evalfunc, esize, csize, skydata, points, vm, **kwargs):
+        fd, rbase = tempfile.mkstemp(dir=f"./", prefix='tmp_eval')
+        rbase = rbase.rsplit("/", 1)[-1]
+        with os.fdopen(fd) as f:
+            pass
+        os.remove(rbase)
+        d = np.linspace(0, len(skydata.maskindices),
+                        int(esize/csize) + 2).astype(int)
+        self.scene.log(self, f"breaking evaluation into {len(d)} chunks,"
+                             f" writing temporary results to {rbase}_XX.npz"
+                       , True)
+        slices = [slice(d[i], d[i + 1], 1) for i in range(len(d) - 1)]
+        omask = np.copy(skydata.mask)
+        for i, slc in enumerate(slices):
+            self.scene.log(self, f"evaluating part {i + 1} of {len(d)}", True)
+            skydata.mask = skydata.maskindices[slc]
+            lr = evalfunc(skydata, points, vm, emax=-1, **kwargs)
+            skydata.mask = omask
+            lr.write(f"{rbase}_{i:02d}.npz")
+        idx = np.arange(len(slices))
+        lrs = [type(lr)(f"{rbase}_{i:02d}.npz") for i in idx]
+        lr = lrs[0].merge(*lrs[1:])
+        [os.remove(f"{rbase}_{i:02d}.npz") for i in idx]
+        return lr
+
     def evaluate(self, skydata, points, vm, viewangle=180.,
                  metricclass=MetricSet, metrics=None, datainfo=False,
                  srconly=False, suntol=10.0, blursun=False, coercesumsafe=False,
-                 stol=10,  minsun=1, **kwargs):
+                 stol=10,  minsun=1, emax=10000, **kwargs):
         """apply sky data and view queries to daylightplane to return metrics
         parallelizes and optimizes run order.
 
@@ -178,6 +205,7 @@ class Integrator(object):
         -------
         raytraverse.lightfield.LightResult
         """
+        # delegate to zonal
         if (points is None and len(self._issunplane) > 0 and
                 np.any(skydata.sun[:, 3] > 0)):
             return self.zonal_evaluate(skydata, self.lightplanes[0].pm, vm,
@@ -185,54 +213,67 @@ class Integrator(object):
                             metrics=metrics, datainfo=datainfo,
                             srconly=srconly, suntol=suntol,
                             blursun=blursun, coercesumsafe=coercesumsafe,
-                            stol=stol,  minsun=minsun,
+                            stol=stol,  minsun=minsun, emax=emax,
                             **kwargs)
         if points is None:
             points, skarea = self._get_fixed_points(self.lightplanes[0].pm)
         else:
             points = np.atleast_2d(points)
+        if emax > 0:
+            esize = len(skydata.maskindices) * len(points)
+        else:
+            esize = 0
+        if 0 < emax < esize:
+            lr = self._echunk(self.evaluate, esize, emax, skydata, points, vm,
+                              viewangle=viewangle, metricclass=metricclass,
+                              metrics=metrics, datainfo=datainfo,
+                              srconly=srconly, suntol=suntol, blursun=blursun,
+                              coercesumsafe=coercesumsafe, stol=stol,
+                              minsun=minsun, **kwargs)
+        else:
+            (vm, vms, cmetrics, ometrics,
+             sumsafe, needs_post) = self._check_params(vm, viewangle, metrics,
+                                                       metricclass,
+                                                       coercesumsafe)
+            tidxs, skydatas, dsns, vecs = self._group_query(skydata, points)
+            oshape = (len(skydata.maskindices), len(points), len(vms),
+                      len(cmetrics))
 
-        (vm, vms, cmetrics, ometrics,
-         sumsafe, needs_post) = self._check_params(vm, viewangle, metrics,
-                                                   metricclass, coercesumsafe)
-        tidxs, skydatas, dsns, vecs = self._group_query(skydata, points)
-        oshape = (len(skydata.maskindices), len(points), len(vms), len(cmetrics))
-
-        self.scene.log(self, f"Evaluating {len(ometrics)} metrics for "
-                             f"{oshape[2]} view directions at {oshape[1]} "
-                             f"points under {oshape[0]} skies", True)
-        fields, isort = self._process_mgr(tidxs, skydatas, dsns,
-                                          self.evaluate_pt,
-                                          message="Evaluating Points",
-                                          srconly=srconly, sumsafe=sumsafe,
-                                          metricclass=metricclass,
-                                          metrics=cmetrics, vm=vm, vms=vms,
-                                          suntol=suntol, blursun=blursun,
-                                          **kwargs)
-        # sort back to original order and reshape
-        fields = np.concatenate(fields, axis=0)[isort].reshape(oshape)
-        if needs_post:
-            fields = self._post_process_metrics(fields, ometrics, cmetrics,
-                                                **kwargs)
-        sinfo, dinfo = self._sinfo(datainfo, vecs, tidxs, oshape[0:2])
-        if sinfo is not None:
-            nshape = list(sinfo.shape)
-            nshape[2] = fields.shape[2]
-            sinfo = np.broadcast_to(sinfo, nshape)
-            fields = np.concatenate((fields, sinfo), axis=-1)
-        # compose axes: (skyaxis, ptaxis, viewaxis, metricaxis)
-        axes = (ResultAxis(skydata.rowlabel[skydata.fullmask], f"sky"),
-                ResultAxis(points, "point"),
-                ResultAxis([v.dxyz for v in vms], "view"),
-                ResultAxis(list(ometrics) + dinfo, "metric"))
-        lr = LightResult(fields, *axes)
+            self.scene.log(self, f"Evaluating {len(ometrics)} metrics for "
+                                 f"{oshape[2]} view directions at {oshape[1]} "
+                                 f"points under {oshape[0]} skies", True)
+            fields, isort = self._process_mgr(tidxs, skydatas, dsns,
+                                              self.evaluate_pt,
+                                              message="Evaluating Points",
+                                              srconly=srconly, sumsafe=sumsafe,
+                                              metricclass=metricclass,
+                                              metrics=cmetrics, vm=vm, vms=vms,
+                                              suntol=suntol, blursun=blursun,
+                                              **kwargs)
+            # sort back to original order and reshape
+            fields = np.concatenate(fields, axis=0)[isort].reshape(oshape)
+            if needs_post:
+                fields = self._post_process_metrics(fields, ometrics, cmetrics,
+                                                    **kwargs)
+            sinfo, dinfo = self._sinfo(datainfo, vecs, tidxs, oshape[0:2])
+            if sinfo is not None:
+                nshape = list(sinfo.shape)
+                nshape[2] = fields.shape[2]
+                sinfo = np.broadcast_to(sinfo, nshape)
+                fields = np.concatenate((fields, sinfo), axis=-1)
+            # compose axes: (skyaxis, ptaxis, viewaxis, metricaxis)
+            axes = (ResultAxis(skydata.rowlabel[skydata.fullmask], f"sky"),
+                    ResultAxis(points, "point"),
+                    ResultAxis([v.dxyz for v in vms], "view"),
+                    ResultAxis(list(ometrics) + dinfo, "metric"))
+            lr = LightResult(fields, *axes)
         return lr
 
     def zonal_evaluate(self, skydata, pm, vm, viewangle=180.,
                        metricclass=MetricSet, metrics=None, srconly=False,
                        suntol=10.0, blursun=False, coercesumsafe=False,
-                       stol=10,  minsun=1, datainfo=False,
-                       **kwargs):
+                       stol=10,  minsun=1, datainfo=False, calcarea=True,
+                       emax=10000, **kwargs):
         """apply sky data and view queries to daylightplane to return metrics
         parallelizes and optimizes run order.
 
@@ -251,51 +292,78 @@ class Integrator(object):
                                  datainfo=datainfo, srconly=srconly,
                                  suntol=suntol, blursun=blursun,
                                  coercesumsafe=coercesumsafe,
-                                 stol=stol, minsun=minsun,
+                                 stol=stol, minsun=minsun, emax=emax,
                                  **kwargs)
+        # only chunk on first call
+        if emax > 0:
+            # call this first just to measure size of evaluation, skip calcarea
+            # for now
+            (sunmask, suns, pts, sundata,
+             skarea) = self._zonal_group_query(skydata, pm, stol=stol,
+                                               minsun=minsun, calcarea=False)
+            sunpts = np.sum([len(p) for p in sundata[0]])
+            skpts = len(pts) * (len(skydata.smtx) - len(sundata[0]))
+            esize = sunpts + skpts
+        else:
+            esize = 0
+        if 0 < emax < esize:
+            lr = self._echunk(self.zonal_evaluate, esize, emax, skydata, pm, vm,
+                              viewangle=viewangle, metricclass=metricclass,
+                              metrics=metrics, datainfo=datainfo,
+                              srconly=srconly, suntol=suntol, blursun=blursun,
+                              coercesumsafe=coercesumsafe, stol=stol,
+                              minsun=minsun, **kwargs)
+        else:
+            (sunmask, suns, pts, sundata,
+             skarea) = self._zonal_group_query(skydata, pm, stol=stol,
+                                               minsun=minsun, calcarea=calcarea)
+            (vm, vms, cmetrics, ometrics,
+             sumsafe, needs_post) = self._check_params(vm, viewangle, metrics,
+                                                       metricclass,
+                                                       coercesumsafe)
+            result = self._unmask_data(skydata.smtx, sunmask, suns, pts,
+                                       sundata, skarea)
+            smtx, dsns, all_vecs, sunidx, serr, areas, cnts = result
 
-        (vm, vms, cmetrics, ometrics,
-         sumsafe, needs_post) = self._check_params(vm, viewangle, metrics,
-                                                   metricclass, coercesumsafe)
-        (tidxs, skydatas, dsns, vecs, serr,
-         areas, pts, cnts) = self._zonal_group_query(skydata, pm,
-                                                     stol=stol, minsun=minsun)
+            tidxs, skydatas = self._match_ragged(smtx, dsns, sunidx, all_vecs)
 
-        self.scene.log(self, f"Evaluating {len(ometrics)} metrics for {len(vms)}"
-                             f" view directions across zone '{pm.name}' under "
-                             f"{len(skydata.sun)} skies", True)
-        fields, isort = self._process_mgr(tidxs, skydatas, dsns,
-                                          self.evaluate_pt,
-                                          message="Evaluating Points",
-                                          srconly=srconly, sumsafe=sumsafe,
-                                          metricclass=metricclass,
-                                          metrics=cmetrics, vm=vm, vms=vms,
-                                          suntol=suntol, blursun=blursun,
-                                          **kwargs)
+            self.scene.log(self, f"Evaluating {len(ometrics)} metrics for "
+                                 f"{len(vms)} view directions across zone "
+                                 f"'{pm.name}' under  {len(skydata.sun)} skies",
+                           True)
+            fields, isort = self._process_mgr(tidxs, skydatas, dsns,
+                                              self.evaluate_pt,
+                                              message="Evaluating Points",
+                                              srconly=srconly, sumsafe=sumsafe,
+                                              metricclass=metricclass,
+                                              metrics=cmetrics, vm=vm, vms=vms,
+                                              suntol=suntol, blursun=blursun,
+                                              **kwargs)
 
-        fields = np.concatenate(fields, axis=0)[isort]
-        if needs_post:
-            fields = self._post_process_metrics(fields, ometrics, cmetrics,
-                                                **kwargs)
-        oshape = (len(fields), len(vms))
-        pmetrics = ['x', 'y', 'z', 'area']
-        if datainfo:
-            sinfo, dinfo = self._zonal_sinfo(serr, tidxs, oshape + (2,))
-            if sinfo is not None:
-                fields = np.concatenate((sinfo, fields), axis=-1)
-                pmetrics += dinfo
+            fields = np.concatenate(fields, axis=0)[isort]
+            if needs_post:
+                fields = self._post_process_metrics(fields, ometrics, cmetrics,
+                                                    **kwargs)
+            oshape = (len(fields), len(vms))
+            pmetrics = ['x', 'y', 'z', 'area']
+            if datainfo:
+                sinfo, dinfo = self._zonal_sinfo(serr, tidxs, oshape + (2,))
+                if sinfo is not None:
+                    fields = np.concatenate((sinfo, fields), axis=-1)
+                    pmetrics += dinfo
 
-        areas = np.broadcast_to(areas[:, None, None], oshape + (1,))
-        axes = [ResultAxis(skydata.rowlabel[skydata.fullmask], f"sky"),
-                ResultAxis([pm.name], f"zone"),
-                ResultAxis([v.dxyz for v in vms], "view"),
-                ResultAxis(pmetrics + ometrics, "metric")]
+            areas = np.broadcast_to(areas[:, None, None], oshape + (1,))
+            axes = [ResultAxis(skydata.rowlabel[skydata.fullmask], f"sky"),
+                    ResultAxis([pm.name], f"zone"),
+                    ResultAxis([v.dxyz for v in vms], "view"),
+                    ResultAxis(pmetrics + ometrics, "metric")]
 
-        strides = np.cumsum(cnts)[:-1]
-        fvecs = np.broadcast_to(vecs[:, None, 3:], oshape + (3,))
-        fields = np.concatenate((fvecs, areas, fields), axis=-1)
-        fields = np.split(fields, strides)
-        return ZonalLightResult(fields, *axes, pointmetrics=pmetrics)
+            strides = np.cumsum(cnts)[:-1]
+            fvecs = np.broadcast_to(all_vecs[:, None, 3:], oshape + (3,))
+            fields = np.concatenate((fvecs, areas, fields), axis=-1)
+            fields = np.split(fields, strides)
+            lr = ZonalLightResult(fields, *axes)
+        return lr
 
     @staticmethod
     def _post_process_metrics(fields, ometrics, cmetrics, **kwargs):
@@ -395,7 +463,7 @@ class Integrator(object):
                                         gshape).ravel())
         return np.stack(idxs), skydatas, dsns, vecs
 
-    def _zonal_group_query(self, skydata, pm, stol=10, minsun=1):
+    def _zonal_group_query(self, skydata, pm, stol=10, minsun=1, calcarea=True):
         pts, skarea = self._get_fixed_points(pm)
         # only look for suns when relevant
         sunmask = skydata.sun[:, 3] > 0
@@ -407,15 +475,13 @@ class Integrator(object):
                                                stol=stol,
                                                minsun=minsun)
         spts = [v[:, 3:] for v in vecs]
-        areas = pool_call(translate.calc_omega, spts, pm, expandarg=False,
-                          desc="calculating areas", pbar=self.scene.dolog)
-
+        if calcarea:
+            areas = pool_call(translate.calc_omega, spts, pm, expandarg=False,
+                              desc="calculating areas", pbar=self.scene.dolog)
+        else:
+            areas = [[0]*len(v) for v in spts]
         sundata = (vecs, idx, ds, areas)
-        result = self._unmask_data(skydata, sunmask, suns, pts, sundata, skarea)
-        smtx, dsns, all_vecs, sunidx, d, areas, cnts = result
-
-        tidxs, skydatas = self._match_ragged(smtx, dsns, sunidx, all_vecs)
-        return tidxs, skydatas, dsns, all_vecs, d, areas, pts, cnts
+        return sunmask, suns, pts, sundata, skarea
 
     def _sinfo(self, datainfo, vecs, idxs, oshape):
         """error and bin information for evaluate queries"""
@@ -447,7 +513,7 @@ class Integrator(object):
                 elif di == "pt_idx":
                     try:
                         ptidx = lp.data.idx[idx, 1]
-                    except IndexError:
+                    except (IndexError, AttributeError):
                         ptidx = idx
                     sinfo.append(ptidx)
                     dinfo2.append(f"{lp.src}_{di}")
@@ -525,7 +591,7 @@ class Integrator(object):
         return skydatas, dsns
 
     @staticmethod
-    def _unmask_data(skydata, sunmask, suns, pts, sundata, skarea):
+    def _unmask_data(dsmtx, sunmask, suns, pts, sundata, skarea):
         # calculate size of return arrays
         cnts = np.full(len(sunmask), len(pts))
         cnts[sunmask] = [len(s) for s in sundata[0]]
@@ -534,7 +600,7 @@ class Integrator(object):
         sunidx = np.full(tcnt, -1, dtype=np.int64)
         d = np.zeros(tcnt)
         areas = np.zeros(tcnt)
-        smtx = np.zeros((tcnt, skydata.smtx.shape[1]))
+        smtx = np.zeros((tcnt, dsmtx.shape[1]))
         dsns = np.zeros((tcnt, suns.shape[1]))
         # track suns (one for each True sunmask
         j = 0
@@ -554,7 +620,7 @@ class Integrator(object):
                 all_vecs[ci:ci + cnt, 0:3] = suns[i, 0:3]
                 all_vecs[ci:ci + cnt, 3:6] = pts
                 areas[ci:ci + cnt] = skarea
-            smtx[ci:ci + cnt] = skydata.smtx[i]
+            smtx[ci:ci + cnt] = dsmtx[i]
             dsns[ci:ci + cnt] = suns[i]
             ci += cnt
         return smtx, dsns, all_vecs, sunidx, d, areas, cnts
@@ -574,21 +640,18 @@ class Integrator(object):
 
         qtup, qidx, tup_isort = self._sort_run_data(tidxs)
 
-        d = np.linspace(0, len(qtup), max(2, int(len(qtup)/250))).astype(int)
-        slices = [slice(d[i], d[i + 1], 1) for i in range(len(d) - 1)]
         self.scene.log(self, f"Calculating {len(qidx)} sun/sky/pt combinations",
                        True)
         if self._sunviewengine is not None:
             eval_kwargs.update(svengine=self._sunviewengine)
-        fields = []
-        for i, slc in enumerate(slices):
-            lptis = []
-            for qi, qt in zip(qidx[slc], qtup[slc]):
-                lptis.append(([(lp.data, tidx[qi]) for lp, tidx
-                               in zip(self.lightplanes, tidxs)], qt))
-            desc = f"{message} ({i+1:02d} of {len(slices):02d})"
-            fields += pool_call(_load_pts, lptis, eval_fn, tidxs.T, mask_kwargs,
-                                skydatas, dsns, desc=desc, pbar=self.scene.dolog, **eval_kwargs)
+
+        lptis = []
+        for qi, qt in zip(qidx, qtup):
+            lptis.append(([(lp.data, tidx[qi]) for lp, tidx
+                           in zip(self.lightplanes, tidxs)], qt))
+        fields = pool_call(_load_pts, lptis, eval_fn, tidxs.T, mask_kwargs,
+                           skydatas, dsns, desc=message, pbar=self.scene.dolog,
+                           **eval_kwargs)
         return fields, tup_isort
 
 
