@@ -7,6 +7,7 @@
 # =======================================================================
 import numpy as np
 from raytraverse import io
+from raytraverse.utility import pool_call
 from scipy.spatial import cKDTree
 
 from raytraverse.lightfield.lightresult import LightResult, ResultAxis
@@ -58,10 +59,7 @@ class ZonalLightResult(LightResult):
 
     """
 
-    def __init__(self, data, *axes, pointmetrics=None):
-        self.pointmetrics = pointmetrics
-        if pointmetrics is None:
-            self.pointmetrics = []
+    def __init__(self, data, *axes):
         super().__init__(data, *axes)
         namee = ["sky", "zone", "view", "metric"]
         if (len(namee) != len(self.names) or
@@ -80,14 +78,13 @@ class ZonalLightResult(LightResult):
     def load(self, file):
         with np.load(file) as result:
             names = result['names']
-            self.pointmetrics = result['pointmetrics']
             axes = tuple([ResultAxis(result[f"arr_{i}"], n)
                           for i, n in enumerate(names)])
             data = [result[f"data_{i}"] for i in range(len(axes[0].values))]
         return data, axes
 
     def write(self, file, compressed=True):
-        kws = dict(names=self.names, pointmetrics=self.pointmetrics)
+        kws = dict(names=self.names)
         for i in range(len(self.data)):
             kws[f"data_{i}"] = self.data[i]
         args = [a.values for a in self.axes]
@@ -101,6 +98,34 @@ class ZonalLightResult(LightResult):
         else:
             self._file = file
 
+    def merge(self, *lrs, axis="sky"):
+        """create merged lightresult from lightresults, must match on all axes
+         except axis. does not sort but culls duplicates"""
+        if axis == "zone":
+            raise ValueError("cannot stack ragged axis zone")
+        outaxes, filters = self._merge_axes(*lrs, axis=axis)
+
+        if axis == "sky":
+            data = list(self.data)
+            for lr, f in zip(lrs, filters):
+                if np.all(f):
+                    data += list(lr.data)
+                else:
+                    data += [i for i, j in zip(lr.data, f) if j]
+        else:
+            oi = self._index(axis) - 1
+            data = []
+            for i in range(len(self.data)):
+                d = self.data[i]
+                for lr, f, in zip(lrs, filters):
+                    if np.all(f):
+                        od = lr.data[i]
+                    else:
+                        od = np.compress(f, lr.data[i], axis=oi)
+                    d = np.concatenate([d, od], axis=oi)
+                data.append(d)
+        return ZonalLightResult(data, *outaxes)
+
     def _pull_labels(self, data, order, preserve, filters):
         ax0_labels = []
         oshp = []
@@ -168,24 +193,47 @@ class ZonalLightResult(LightResult):
 
     def pull2hdr(self, imgzone, basename, showsample=False, **kwargs):
         pm = PlanMapper(imgzone)
-        zimg, vecs, mask, mask2, header = pm.init_img(480)
         if "metric" in kwargs and kwargs["metric"] is not None:
             kwargs["metric"] = np.unique(np.concatenate(([0, 1, 2], kwargs["metric"])))
         rt, labels, names = self.pull("metric", preserve=2, **kwargs)
         flabels0 = self.fmt_names(names[-1], labels[-1])
         flabels1 = self.fmt_names(names[-2], labels[-2][3:])
-        for i, la0 in enumerate(flabels0):
-            data = rt[i]
-            kd = cKDTree(data[:, 0:3])
-            data = data[:, 3:]
-            err, idx = kd.query(vecs[mask])
-            for k, (la, d) in enumerate(zip(flabels1, data.T)):
-                img = np.copy(zimg)
-                img[mask] = d[idx]
-                if showsample:
-                    if len(img.shape) < 3:
-                        img = np.repeat(img[None, ...], 3, 0)
-                    img = pm.add_vecs_to_img(img, kd.data)
-                    io.carray2hdr(img, f"{basename}_{la}_{la0}.hdr", [header])
-                else:
-                    io.array2hdr(img, f"{basename}_{la}_{la0}.hdr", [header])
+        pool_call(_pull2hdr, list(zip(rt, flabels0)), flabels1, pm, basename,
+                  showsample=showsample)
+
+    pull2planhdr = pull2hdr
+
+    def rebase(self, points):
+        paxis = ResultAxis(points, "point")
+        omet = self.axis("metric").values
+        mf = [i for i, v in enumerate(omet) if v not in
+              ('x', 'y', 'z', 'area')]
+        maxis = ResultAxis([omet[i] for i in mf] + ["rebase_err"], "metric")
+        odata = pool_call(_pull2grid, self.data, points, mf, expandarg=False)
+        lr = LightResult(np.stack(odata), self.axes[0], paxis, self.axes[2], maxis)
+        return lr
+
+
+def _pull2grid(data, points, mf):
+    kd = cKDTree(data[:, 0, 0:3])
+    err, idx = kd.query(points)
+    odata = data[idx][..., mf]
+    oerr = np.broadcast_to(err[:, None, None], (*odata.shape[:-1], 1))
+    return np.concatenate((odata, oerr), axis=-1)
+
+
+def _pull2hdr(data, la0, flabels1, pm, basename, showsample=False):
+    zimg, vecs, mask, mask2, header = pm.init_img(480)
+    kd = cKDTree(data[:, 0:3])
+    data = data[:, 3:]
+    err, idx = kd.query(vecs[mask])
+    for k, (la, d) in enumerate(zip(flabels1, data.T)):
+        img = np.copy(zimg)
+        img[mask] = d[idx]
+        if showsample:
+            if len(img.shape) < 3:
+                img = np.repeat(img[None, ...], 3, 0)
+            img = pm.add_vecs_to_img(img, kd.data)
+            io.carray2hdr(img, f"{basename}_{la}_{la0}.hdr", [header])
+        else:
+            io.array2hdr(img, f"{basename}_{la}_{la0}.hdr", [header])
